# -*- coding: utf-8 -*-

"""Calculates the current version number.

If possible, uses output of “git describe” modified to conform to the
visioning scheme that setuptools uses (see PEP 386).  Releases must be
labelled with annotated tags (signed tags are annotated) of the following
format:

   v<num>(.<num>)+ [ {a|b|c|rc} <num> (.<num>)* ]

If “git describe” returns an error (likely because we're in an unpacked copy
of a release tarball, rather than a git working copy), or returns a tag that
does not match the above format, version is read from RELEASE-VERSION file.

To use this script, simply import it your setup.py file, and use the results
of get_version() as your package version:

    import version
    setup(
        version=version.get_version(),
        .
        .
        .
    )

This will automatically update the RELEASE-VERSION file.  The RELEASE-VERSION
file should *not* be checked into git but it *should* be included in sdist
tarballs (as should version.py file).  To do this, run:

    echo include RELEASE-VERSION version.py >>MANIFEST.in
    echo RELEASE-VERSION >>.gitignore

With that setup, a new release can be labelled by simply invoking:

    git tag -s v1.0
"""

__author__ = ("Douglas Creager <dcreager@dcreager.net>", "Michal Nazarewicz <mina86@mina86.com>")
__license__ = "This file is placed into the public domain."
__maintainer__ = "Michal Nazarewicz"
__email__ = "mina86@mina86.com"

__all__ = "get_version"


import re
import subprocess
import sys


RELEASE_VERSION_FILE = "RELEASE-VERSION"

# http://www.python.org/dev/peps/pep-0386/
_PEP386_SHORT_VERSION_RE = r"\d+(?:\.\d+)+(?:(?:[abc]|rc)\d+(?:\.\d+)*)?"
_PEP386_VERSION_RE = r"^%s(?:\.post\d+)?(?:\.dev\d+)?$" % (_PEP386_SHORT_VERSION_RE)
_GIT_DESCRIPTION_RE = r"^v(?P<ver>%s)-(?P<commits>\d+)-g(?P<sha>[\da-f]+)$" % (
    _PEP386_SHORT_VERSION_RE
)


def read_git_version():
    try:
        proc = subprocess.Popen(  # pylint: disable=R1732
            ("git", "describe", "--long", "--tags", "--match", "v[0-9]*.*"),
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        data, _ = proc.communicate()
        if proc.returncode:
            return None
        ver = data.decode().splitlines()[0].strip()
    except:
        return None

    if not ver:
        return None
    match = re.search(_GIT_DESCRIPTION_RE, ver)
    if not match:
        sys.stderr.write("version: git description (%s) is invalid, " "ignoring\n" % ver)
        return None

    commits = int(match.group("commits"))
    if not commits:
        return match.group("ver")
    return "%s.post%d.dev%d" % (match.group("ver"), commits, int(match.group("sha"), 16))


def read_release_version():
    try:
        with open(RELEASE_VERSION_FILE) as infile:
            ver = infile.readline().strip()
        if not re.search(_PEP386_VERSION_RE, ver):
            sys.stderr.write(
                "version: release version (%s) is invalid, " "will use it anyway\n" % ver
            )
        return ver
    except:
        return None


def write_release_version(version):
    with open(RELEASE_VERSION_FILE, "w") as outfile:
        outfile.write("%s\n" % version)


def get_version():
    release_version = read_release_version()
    version = read_git_version() or release_version
    if not version:
        raise ValueError("Cannot find the version number")
    if version != release_version:
        write_release_version(version)
    return version


if __name__ == "__main__":
    print(get_version())



#!/usr/bin/env python3
"""
Creates a .dbi helper file to accelerate web access for an immutable SQLite database.

Scans the database file to identify key pages that readers will use frequently:
    - first page
    - SQLite system tables
    - btree interior pages for all tables and indexes
Copies these into a .dbi file, itself a small SQLite database (typically <1% of the full database,
YMMV), indexed by their original offsets.

Those key pages are typically scattered all throughout the main database file (even after vacuum).
Therefore, prefetching them by downloading the compact .dbi file can significantly streamline the
subsequent random access patterns needed to answer queries.

Assuming both the main database and .dbi files are immutable, the consistency between them can be
verified, within reason, by checking that the SQLite header (first 100 bytes of main database file)
matches that stored in the .dbi file pageno=1.

Background:
    https://www.sqlite.org/fileformat.html
    https://sqlite.org/dbstat.html
"""

import sys
import os
import argparse
import itertools
import contextlib
import multiprocessing
import sqlite3


def main(argv):
    parser = argparse.ArgumentParser(
        prog=os.path.basename(argv[0]),
        description="Create .dbi index file to accelerate web access for an immutable SQLite database",
    )
    parser.add_argument(
        "dbfile",
        type=str,
        metavar="DBFILE",
        help="immutable SQLite database filename"
        if __package__ != "genomicsqlite"
        else argparse.SUPPRESS,
    )
    parser.add_argument(
        "-o",
        dest="dbifile",
        type=str,
        default=None,
        help="output .dbi filename (default: DBFILE.dbi)",
    )
    parser.add_argument(
        "-f", "--force", action="store_true", help="remove existing output .dbi file, if any"
    )
    args = parser.parse_args(argv[1:])

    # read basic db info
    header = read_db_header(args.dbfile)
    dbfile_mtime = os.stat(args.dbfile).st_mtime
    dbfile_size = os.path.getsize(args.dbfile)
    page_size, btrees = read_db_btrees(args.dbfile)
    assert not (dbfile_size % page_size)
    last_pageno = dbfile_size // page_size

    if not btrees:
        print(f"[WARN] database appears to be empty", file=sys.stderr)
    if not args.dbifile:
        args.dbifile = args.dbfile + ".dbi"
    if os.path.exists(args.dbifile):
        if os.path.isfile(args.dbifile) and args.force:
            print(f"[WARN] deleting existing {args.dbifile}", file=sys.stderr)
            os.unlink(args.dbifile)
        else:
            print(f"delete existing {args.dbifile} and try again", file=sys.stderr)
            sys.exit(1)
    if os.path.basename(args.dbifile) != os.path.basename(args.dbfile) + ".dbi":
        print(
            f"[WARN] index filename should be {os.path.basename(args.dbfile)}.dbi", file=sys.stderr
        )
    print(
        f"indexing for web access: {args.dbfile} ({os.path.getsize(args.dbfile):,} bytes)",
        file=sys.stderr,
    )
    print(f"page size: {page_size:,} bytes", file=sys.stderr)
    print(f"scanning: {len(btrees):,} btrees", file=sys.stderr)
    sys.stderr.flush()

    # identify the desired pages
    desired_pagenos = collect_pagenos(args.dbfile, btrees)
    # spike in the first & last 64 KiB worth of pages
    for i in range(1, min(last_pageno, 65536 // page_size)):
        desired_pagenos.add(i)
    for i in range(max(1, last_pageno - 65536 // page_size + 1), last_pageno + 1):
        desired_pagenos.add(i)
    print(f"pages to copy: {len(desired_pagenos):,}", file=sys.stderr)
    sys.stderr.flush()

    try:
        # write .dbi file
        write_dbi(args.dbfile, page_size, btrees, desired_pagenos, args.dbifile)
        # check database file hasn't been modified
        assert (
            os.stat(args.dbfile).st_mtime == dbfile_mtime
            and os.path.getsize(args.dbfile) == dbfile_size
            and read_db_header(args.dbfile) == header
        ), "Database file was concurrently modified"
    except:
        try:
            os.unlink(args.dbifile)
        except:
            print(f"[WARN] failed to remove {args.dbifile} upon error", file=sys.stderr)
        raise

    print(f"wrote: {args.dbifile} ({os.path.getsize(args.dbifile):,} bytes)", file=sys.stderr)


def read_db_header(dbfile):
    "Read the SQLite database file header (first 100 bytes)"
    with open(dbfile, "rb") as infile:
        header = infile.read(100)
    assert (
        isinstance(header, bytes) and len(header) == 100 and header[:16] == b"SQLite format 3\000"
    ), "Unable to read SQLite database header"
    return header


def read_db_btrees(dbfile):
    "List the table and index names"
    dbh = sqlite3.connect(dbfile)

    # detect a common failure mode (SQLite3 not built with dbstat)
    try:
        next(dbh.execute("select pageno from dbstat limit 1"))
    except sqlite3.OperationalError as exn:
        if "no such table: dbstat" in str(exn):
            raise RuntimeError(
                "This tool requires SQLite3 to have been built with SQLITE_ENABLE_DBSTAT_VTAB"
            )
        else:
            raise

    page_size = next(dbh.execute("pragma page_size"))[0]
    btrees = [row[0] for row in dbh.execute("select name from sqlite_master")]
    dbh.close()
    assert page_size in (512, 1024, 2048, 4096, 8192, 16384, 32768, 65536)
    btrees.append("sqlite_master")
    return page_size, btrees


def collect_pagenos(dbfile, btrees):
    "List the interior page numbers for all the given table & index names"
    with multiprocessing.Pool(min(8, multiprocessing.cpu_count())) as pool:
        return set(
            itertools.chain(
                *pool.map(collect_pagenos_worker, [(dbfile, btree) for btree in btrees])
            )
        )


def collect_pagenos_worker(inp):
    dbfile = inp[0]
    btree = inp[1]
    dbh = sqlite3.connect(dbfile)
    # ref: https://github.com/sqlite/sqlite/blob/master/src/dbstat.c
    query = "select pageno from dbstat where name=?"
    if not (
        btree.startswith("sqlite_")
        or btree.endswith("pages_btree_interior")
        or btree.endswith("pages_btree_interior_pageno")
    ) or btree.startswith("sqlite_autoindex_"):
        # include all pages of sqlite system tables, and sqlite_zstd_vfs' index of its own nested
        # btree interior pages
        query += " and pagetype='internal'"
    pagenos = [row[0] for row in dbh.execute(query, (btree,))]
    dbh.close()
    return pagenos


def write_dbi(dbfile, page_size, btrees, pagenos, dbifile):
    "generate .dbi file, copying the content of the given page numbers"

    with contextlib.ExitStack() as stack:
        cursor = open(dbfile, "rb")
        stack.enter_context(cursor)
        cursor.seek(0, os.SEEK_END)
        dbfile_size = cursor.tell()
        assert dbfile_size and not (dbfile_size % page_size)

        assert not os.path.exists(dbifile), "remove existing .dbi file " + dbifile
        dbih = sqlite3.connect(dbifile)
        stack.callback(lambda dbih: dbih.close(), dbih)

        # application_id = "dbi"
        # small page size minimizes bin-packing overhead to store blobs of comparable size
        dbih.executescript(
            """
            pragma application_id = 0x646269;
            pragma page_size = 1024;
            pragma locking_mode = EXCLUSIVE
            """
        )

        with dbih:  # transaction
            dbih.executescript(
                """
                create table web_dbi_meta(key text primary key, value blob) without rowid;
                create table web_dbi_pages(offset integer primary key, data blob not null)
                """
            )

            for pageno in pagenos:
                assert 0 < pageno <= (dbfile_size // page_size)
                ofs = (pageno - 1) * page_size
                assert ofs + page_size <= dbfile_size
                cursor.seek(ofs)
                pagedata = cursor.read(page_size)
                assert len(pagedata) == page_size
                dbih.execute("insert into web_dbi_pages(offset,data) values(?,?)", (ofs, pagedata))

            for kv in [("page_size", page_size), ("dbfile_size", dbfile_size)] + [
                ("main." + btree, None) for btree in btrees
            ]:
                dbih.execute("insert into web_dbi_meta(key,value) values(?,?)", kv)

        dbih.executescript("vacuum")


if __name__ == "__main__":
    main(sys.argv)



#!/usr/bin/env python3
"""
GenomicSQLite Python binding
"""
import os
import sys
import platform
import json
import sqlite3
from typing import Optional, NamedTuple, Dict, Any

_HERE = os.path.dirname(__file__)
_YES = ("1", "true", "t", "yes", "y")

# Module initialization -- locate shared library file
_DLL = os.environ.get("LIBGENOMICSQLITE", "").strip()  # use env var if present
if not _DLL and platform.machine() == "x86_64":
    # select platform-appropriate library bundled with this package
    _DLL = {"Linux": ".so", "Darwin": ".dylib"}.get(platform.system(), None)
    if _DLL:
        _DLL = os.path.join(_HERE, "libgenomicsqlite") + _DLL
        if not os.path.isfile(_DLL):
            _DLL = None
if not _DLL:
    # otherwise, let SQLite use dlopen() to look for it
    _DLL = "libgenomicsqlite"
# open a dummy connection to :memory: just for loading the extension.
_MEMCONN = sqlite3.connect(":memory:")
_MEMCONN.enable_load_extension(True)
_MEMCONN.load_extension(_DLL)
# now that it's been loaded, the extension will automatically enable itself on any new connections.


def _execute1(conn, sql, params=None):
    """
    conn.execute(sql,params) and return row 1 column 1
    """
    assert isinstance(conn, sqlite3.Connection)
    return next(conn.execute(sql, params) if params else conn.execute(sql))[0]


__version__ = _execute1(_MEMCONN, "SELECT genomicsqlite_version()")

# load default configuration
_DEFAULT_CONFIG = json.loads(_execute1(_MEMCONN, "SELECT genomicsqlite_default_config_json()"))


def connect(dbfile: str, read_only: bool = False, **kwargs) -> sqlite3.Connection:
    """
    Open a SQLite connection & activate GenomicSQLite extension on it
    """

    # kwargs may be a mix of GenomicSQLite config settings and sqlite3.connect() kwargs. Extract
    # the GenomicSQLite settings based on the keys found in the default configuration.
    kwargs = dict(kwargs)
    config = {}
    for config_key in _DEFAULT_CONFIG:
        if config_key in kwargs:
            config[config_key] = kwargs[config_key]
            del kwargs[config_key]
    if read_only:
        config["mode"] = "ro"
    config_json = json.dumps(config)

    # formulate the URI connection string
    uri = _execute1(
        _MEMCONN,
        "SELECT genomicsqlite_uri(?,?)",
        (dbfile, config_json),
    )

    # open the connection
    conn = sqlite3.connect(uri, uri=True, **kwargs)

    # perform GenomicSQLite tuning
    tuning_sql = _execute1(
        conn,
        "SELECT genomicsqlite_tuning_sql(?)",
        (config_json,),
    )
    conn.executescript(tuning_sql)

    return conn


def attach_sql(conn: sqlite3.Connection, dbfile: str, schema_name: str, **config) -> str:
    config_json = json.dumps(config)
    return _execute1(
        conn, "SELECT genomicsqlite_attach_sql(?,?,?)", (dbfile, schema_name, config_json)
    )


def vacuum_into_sql(conn: sqlite3.Connection, destfile: str, **config) -> str:
    config_json = json.dumps(config)
    return _execute1(conn, "SELECT genomicsqlite_vacuum_into_sql(?,?)", (destfile, config_json))


def create_genomic_range_index_sql(
    conn: sqlite3.Connection,
    table: str,
    rid: str,
    beg: str,
    end: str,
    floor: Optional[int] = None,
) -> str:
    return _execute1(
        conn, "SELECT create_genomic_range_index_sql(?,?,?,?,?)", (table, rid, beg, end, floor)
    )


def genomic_range_rowids_sql(
    conn: sqlite3.Connection,
    indexed_table: str,
    qrid: Optional[str] = None,
    qbeg: Optional[str] = None,
    qend: Optional[str] = None,
    ceiling: Optional[int] = None,
    floor: Optional[int] = None,
) -> str:
    return _execute1(
        conn,
        "SELECT genomic_range_rowids_sql(?,?,?,?,?,?)",
        (indexed_table, qrid, qbeg, qend, ceiling, floor),
    )


def put_reference_assembly_sql(
    conn: sqlite3.Connection, assembly: str, schema: Optional[str] = None
) -> str:
    return _execute1(conn, "SELECT put_genomic_reference_assembly_sql(?,?)", (assembly, schema))


def put_reference_sequence_sql(
    conn: sqlite3.Connection,
    name: str,
    length: int,
    assembly: Optional[str] = None,
    refget_id: Optional[str] = None,
    meta: Optional[Dict[str, Any]] = None,
    rid: Optional[int] = None,
    schema: Optional[str] = None,
):
    meta_json = json.dumps(meta) if meta else None
    return _execute1(
        conn,
        "SELECT put_genomic_reference_sequence_sql(?,?,?,?,?,?,?)",
        (name, length, assembly, refget_id, meta_json, rid, schema),
    )


class ReferenceSequence(NamedTuple):
    rid: int
    name: str
    length: int
    assembly: Optional[str]
    refget_id: Optional[str]
    meta: Dict[str, Any]


def get_reference_sequences_by_rid(
    con: sqlite3.Connection, assembly: Optional[str] = None, schema: Optional[str] = None
) -> Dict[int, ReferenceSequence]:
    table = "_gri_refseq"
    if schema:
        table = f"{schema}.{table}"
    sql = (
        "SELECT _gri_rid, gri_refseq_name, gri_refseq_length, gri_assembly, gri_refget_id, gri_refseq_meta_json FROM "
        + table
    )
    params = []
    if assembly:
        sql += " WHERE gri_assembly = ?"
        params = (assembly,)
    ans = {}
    for row in con.execute(sql, params):
        assert (
            isinstance(row[0], int) and row[0] not in ans
        ), "genomicsqlite: invalid or duplicate reference sequence rid"
        ans[row[0]] = ReferenceSequence(
            rid=row[0],
            name=row[1],
            length=row[2],
            assembly=row[3],
            refget_id=row[4],
            meta=(json.loads(row[5]) if row[5] else {}),
        )
    return ans


def get_reference_sequences_by_name(
    con: sqlite3.Connection, assembly: Optional[str] = None, schema: Optional[str] = None
) -> Dict[str, ReferenceSequence]:
    ans = {}
    for _, item in get_reference_sequences_by_rid(con, assembly, schema).items():
        assert item.name not in ans, "genomicsqlite: non-unique reference sequence names"
        ans[item.name] = item
    return ans


_USAGE = """
Enters the sqlite3 command-line shell on a GenomicSQLite-compressed database.

sqlite3_ARG: passed through to sqlite3 (see `sqlite3 -help`)

--

Other tools:

genomicsqlite DB_FILENAME --compact [options|--help]
[Re]compress and defragment an existing SQLite3 or GenomicSQLite database. See `genomicsqlite --compact --help`

genomicsqlite DB_FILENAME --dbi [options|--help]
Generate a .dbi helper file to optimize web access for a given database. See `genomicsqlite --dbi --help`

"""


def _cli(argv=None):
    """
    Command-line entry point wrapping the `sqlite3` interactive CLI to open a GenomicSQLite
    compressed database file.

    This isn't part of the Python-facing GenomicSQLite API, and shouldn't be reimplemented in other
    language bindings.
    """

    if argv is None:
        argv = sys.argv
    if "--compact" in argv or "-compact" in argv:
        _compact(argv[1], argv[2:])
        return
    if "--dbi" in argv or "-dbi" in argv:
        _dbi(argv[1], argv[2:])
        return

    if (
        len(argv) < 2
        or not (
            next((True for pfx in ("http:", "https:") if argv[1].startswith(pfx)), False)
            or os.path.isfile(argv[1])
        )
        or next((True for a in ("-h", "-help", "--help") if a in argv), False)
    ):
        print(
            "Usage: genomicsqlite DB_FILENAME [--readonly] [sqlite3_ARG ...]\n",
            file=sys.stderr,
        )
        print(
            _USAGE.strip(),
            file=sys.stderr,
        )
        sys.exit(1)

    import shutil

    if not shutil.which("sqlite3"):
        print(
            "Error: the `sqlite3` command-line shell doesn't seem to be available; ensure it's installed and found in $PATH",
            file=sys.stderr,
        )
        sys.exit(1)

    cfg = {}
    try:
        argv[argv.index("--readonly")] = "-readonly"
    except ValueError:
        pass
    if "-readonly" in argv:
        cfg["mode"] = "ro"
    cfg = json.dumps(cfg)
    uri = _execute1(_MEMCONN, "SELECT genomicsqlite_uri(?,?)", (argv[1], cfg))
    tuning_sql = _execute1(_MEMCONN, "SELECT genomicsqlite_tuning_sql(?)", (cfg,))

    cmd = [
        "sqlite3",
        "-bail",
        "-cmd",
        f".load {_DLL}",
        "-cmd",
        f".open {uri}",
        "-cmd",
        ".once /dev/null",
        "-cmd",
        tuning_sql,
        "-cmd",
        '.prompt "GenomicSQLite> "',
    ]
    if sys.stdout.isatty() and not next(
        (arg for arg in argv[2:] if not arg.startswith("-")), False
    ):
        # interactive mode:
        cmd += [
            "-cmd",
            'SELECT "GenomicSQLite " || genomicsqlite_version()',
            "-cmd",
            ".headers on",
        ]
    cmd.append(":memory:")  # placeholder so remaining positional args are recognized as such
    cmd += argv[2:]

    if "DEBUG" in os.environ:
        print(
            " ".join(
                (
                    (
                        (arg if " " not in arg else f"'{arg}'")
                        + (" \\\n    " if len(arg) > 50 else "")
                    )
                    for arg in cmd
                )
            )
        )
    os.execvp("sqlite3", cmd)


def _compact(dbfilename, argv):
    import argparse
    import urllib.parse

    parser = argparse.ArgumentParser(
        prog="genomicsqlite DB_FILENAME --compact",
        description="[Re]compress and defragment an existing SQLite3 or GenomicSQLite database.",
    )
    parser.add_argument("-compact", "--compact", action="store_true", help=argparse.SUPPRESS)
    parser.add_argument(
        "-o", dest="out_filename", type=str, help="compacted database path [DB_FILENAME.compact]"
    )
    parser.add_argument(
        "-l",
        "--level",
        dest="zstd_level",
        metavar="N",
        type=int,
        default=6,
        choices=range(-5, 20),
        help="compression level (-5 to 19) [6]",
    )
    parser.add_argument(
        "--inner-page-KiB",
        type=int,
        choices=(1, 2, 4, 8, 16, 32, 64),
        default=16,
        help="inner page size [16]",
    )
    parser.add_argument(
        "--outer-page-KiB",
        type=int,
        choices=(1, 2, 4, 8, 16, 32, 64),
        default=64,
        help="outer page size [64]",
    )
    parser.add_argument(
        "--no-defrag", action="store_true", help="skip defragmentation of compressed data"
    )
    parser.add_argument(
        "-@", dest="threads", type=int, default=-1, help="worker threads [host CPUs up to 8]"
    )
    parser.add_argument(
        "-f",
        dest="force",
        action="store_true",
        help="overwrite existing file at target path, if any",
    )
    parser.add_argument("-q", dest="quiet", action="store_true", help="suppress progress messages")
    args = parser.parse_args(argv)
    cfg = {"page_cache_MiB": 64}
    # reduced page_cache_MiB: memory usage goes unexpectedly high otherwise. Theory: SQLite
    # miscalculates the size of the page cache for the new database file when we use VACUUM INTO,
    # maybe due to some confusion involving the cache_size & page_size of the two database files.
    for k in ("zstd_level", "inner_page_KiB", "outer_page_KiB", "threads"):
        cfg[k] = vars(args)[k]

    # sniff whether input db is currently compressed with zstd_vfs or not
    con = None
    web = dbfilename.startswith("http:") or dbfilename.startswith("https:")
    if not web:
        con = sqlite3.connect(f"file:{urllib.parse.quote(dbfilename)}?mode=ro", uri=True)
        if next(con.execute("PRAGMA application_id"))[0] != 0x7A737464:
            # db is uncompressed: apply tuning PRAGMAs
            if not args.quiet:
                print(f"Opened uncompressed database {dbfilename}", file=sys.stderr)
                sys.stderr.flush()
            con.executescript(
                _execute1(
                    con,
                    "SELECT genomicsqlite_tuning_sql(?)",
                    (json.dumps(cfg),),
                )
            )
        else:
            # db is zstd_vfs outer db; proceed to open using our own connect()
            con.close()
            con = None
    if not con:
        if not args.quiet:
            print(f"Opening compressed database {dbfilename} ...", file=sys.stderr)
            sys.stderr.flush()
        con = connect(dbfilename, read_only=True, **cfg)

    # VACUUM INTO to [re]compress
    destfilename = args.out_filename
    if not destfilename:
        destfilename = dbfilename
        if web:
            destfilename = os.path.basename(destfilename)
            try:
                destfilename = destfilename[: destfilename.index("?")]
            except ValueError:
                pass
            assert destfilename
        destfilename += ".compact"

    if not args.quiet:
        print(f"Compressing into {destfilename} ...", file=sys.stderr)
        sys.stderr.flush()
    if args.force and os.path.isfile(destfilename):
        os.unlink(destfilename)
    con.executescript(vacuum_into_sql(con, destfilename, **cfg))
    con.close()

    # VACUUM compressed outer db
    if not args.no_defrag:
        if not args.quiet:
            print("Defragmenting compressed data...", file=sys.stderr)
            sys.stderr.flush()
        con = sqlite3.connect(destfilename, uri=False)
        con.executescript(
            "PRAGMA journal_mode=OFF; PRAGMA synchronous=OFF; PRAGMA locking_mode=EXCLUSIVE"
        )
        con.executescript("VACUUM")

    con.close()
    if not args.quiet:
        print("OK", file=sys.stderr)


def _dbi(dbfilename, argv):
    from .sqlite_web_dbi import main as dbi_main  # pylint: disable=E0401,E0611

    argv = [elt for elt in argv if elt not in ("-dbi", "--dbi")]
    dbi_main(["genomicsqlite DBFILE --dbi", dbfilename] + argv)


if __name__ == "__main__":
    _cli(sys.argv)



#!/usr/bin/env python3
from os import path
from setuptools import setup, find_packages
from version import get_version

HERE = path.abspath(path.dirname(__file__))
with open(path.join(HERE, "README.md")) as f:
    README = f.read()

setup(
    python_requires=">=3.6",
    name="genomicsqlite",
    version=get_version(),
    url="https://github.com/mlin/GenomicSQLite",
    description="Genomics Extension for SQLite",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Mike Lin",
    author_email="dna@mlin.net",
    license="Apache",
    packages=find_packages(),
    package_data={
        "": ["genomicsqlite/libgenomicsqlite.so", "genomicsqlite/libgenomicsqlite.dylib"]
    },
    include_package_data=True,
    entry_points={"console_scripts": ["genomicsqlite = genomicsqlite:_cli"]},
)
